﻿/// Copyright (c) Taoffi Nassar, 2009-2010. All Rights Reserved.
/// http://www.isosoft.org/taoffi
///
/// This code is provided as is for sample demonstration purpose
/// 
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Data.OleDb;

/// <summary>
/// Summary description for DataRow
/// </summary>
[DataContract]
public class SilverDataRow
{

	protected SilverDataTable			m_parent_table		= null;
	protected List<SilverDataCell>		m_cells				= new List<SilverDataCell>();
	
	public SilverDataRow()
	{
	}

	public SilverDataRow(SilverDataTable parent_table)
	{
		ParentTable	= parent_table;
	}

	public SilverDataRow( SilverDataTable parent_table, OleDbDataReader data_reader)
	{
		ParentTable		= parent_table;
		ReadDataReader( data_reader);
	}


	[DataMember]
	public List<SilverDataCell> Cells
	{
		get
		{
			if( m_cells == null)
				m_cells = new List<SilverDataCell>();

			return m_cells;
		}
		set
		{
			if (value == null)
				return;
			m_cells = value;
		}
	}


	public void Clear()
	{
		if( m_cells == null)
			return;
		m_cells.Clear();
	}


	public int Count
	{
		get { return (m_cells == null) ?(0) :(m_cells.Count); }
	}

	public SilverDataCell this[string column_name]
	{
		get
		{
			if( string.IsNullOrEmpty( column_name) || Count <= 0)
				return null;
			foreach (SilverDataCell c in m_cells)
			{
				if( string.Compare( c.ColumnName, column_name, true) == 0)
					return c;
			}
			return null;
		}
	}

	public SilverDataTable ParentTable
	{
		get { return m_parent_table; }
		set { m_parent_table = value; }
	}

	public void Add(SilverDataCell cell)
	{
		if( cell == null)
			return;

		SilverDataCell existing = this[cell.ColumnName];

		if( existing == null)
			m_cells.Add( new SilverDataCell( cell));
		else
			existing.CopyOther(cell);
	}

	public bool ReadDataReader(OleDbDataReader data_reader)
	{
		if( data_reader == null || data_reader.IsClosed || m_parent_table == null)
			return false;

		SilverMetaTable		meta_table	= m_parent_table.MetaDataTable;

		if( meta_table == null)
			return false;

		Clear();

		int					n_fields	= data_reader.FieldCount;
		string				field_name,
							field_value;
		SilverMetaColumn	meta_column;
		Type				field_type;

		if( n_fields <= 0)
			return false;

		for( int ndx = 0; ndx < n_fields; ndx++)
		{
			field_name		= data_reader.GetName( ndx);

			meta_column		= meta_table[field_name];
			if( meta_column == null)
				continue;

			field_type		= data_reader.GetFieldType(ndx);

			if( data_reader.IsDBNull( ndx))
				field_value	= "";
			else
				field_value	= data_reader.GetValue(ndx).ToString();

			m_cells.Add(new SilverDataCell(this, meta_column, field_value));
		}

		return true;
	}
}
