﻿/// Copyright (c) Taoffi Nassar, 2009-2010. All Rights Reserved.
/// http://www.isosoft.org/taoffi
///
/// This code is provided as is for sample demonstration purpose
/// 
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Data.OleDb;
using System.Data;

/// <summary>
/// Summary description for SilverDataTable
/// </summary>
[DataContract]
public class SilverDataTable
{
	protected int					m_max_records		= 50;

	protected string				m_table_name		= "",
									m_sql_command		= "";

	protected SilverMetaTable		m_meta_data_table	= new SilverMetaTable();
	protected DbConnectionStatus	m_status			= new DbConnectionStatus();

	protected List<SilverDataRow>	m_rows			= new List<SilverDataRow>();


	public SilverDataTable()
	{
	}

	public SilverDataTable(string table_name)
	{
	}

	public int MaxPageRecords
	{
		get { return m_max_records; }
		set { m_max_records = value; }
	}

	[DataMember]
	public string TableName
	{
		get { return m_table_name; }
		set { m_table_name = value; }
	}

	[DataMember]
	public string UserDefinedSqlCommand
	{
		get { return m_sql_command; }
		set { m_sql_command = value; }
	}

	[DataMember]
	public DbConnectionStatus Status
	{
		get { return m_status; }
		set
		{
			if( value == null)
				return;

			m_status = value;
		}
	}

	[DataMember]
	public List<SilverDataRow> Rows
	{
		get
		{
			if( m_rows == null)
				m_rows = new List<SilverDataRow>();

			return m_rows;
		}
		set
		{
			if( value == null)
				return;
			m_rows = value;
		}
	}

	[DataMember]
	public string ErrorMessage
	{
		get { return (m_status == null) ? ("") : (m_status.ErrorMessage); }
		set
		{

		}
	}


	public bool IsUserDefinedSqlCommand
	{
		get { return (string.IsNullOrEmpty(m_sql_command)) ? (false) : (true); }
	}


	[DataMember]
	public string SqlCommand
	{
		get
		{
			if( IsUserDefinedSqlCommand)
				return m_sql_command;

			if( string.IsNullOrEmpty( m_table_name))
				return "";
			return "SELECT * FROM " + m_table_name;
		}
		set { SqlCommand = value; }
	}



	[DataMember]
	public SilverMetaTable MetaDataTable
	{
		get { return m_meta_data_table; }
		set
		{
			if( value ==null)
				return;
			m_meta_data_table	= value;
		}
	}

	public int Count
	{
		get { return (m_rows == null) ? (0) : (m_rows.Count); }
	}

	public SilverDataRow this[int index]
	{
		get
		{
			if( Count <= 0 ||index >= Count)
				return null;

			return m_rows[index];
		}
	}

	//public void Add(SilverDataRow row)
	//{
	//    if( row == null)
	//        return;

	//    Rows.Add( row);
	//}



	public bool UpdateSchema(string str_connect_string)
	{
		OleDbConnection	conn	= OpenConnection( str_connect_string);
		if( conn == null)
			return false;

		string str_cmd = SqlCommand;

		if (string.IsNullOrEmpty(str_cmd))
		{
			m_status.ErrorMessage = "No data table or sql command specified.";
			return false;
		}

		OleDbDataAdapter	adapter	= new OleDbDataAdapter(str_cmd, conn);
		DataTable			table	= new DataTable();

		try
		{
			table	= adapter.FillSchema(table, SchemaType.Mapped);
		}
		catch (Exception ex)
		{
			m_status.CurrentException = ex;
			return false;
		}
		finally
		{
			conn.Close();
			conn.Dispose();
			OleDbConnection.ReleaseObjectPool();
		}

		if (table == null)
		{
			m_status.ErrorMessage = "Error, couldn't obtain schema table.";
			return false;
		}

		MetaDataTable.ReadDbTableColumns( table);
		return true;
	}


	protected OleDbConnection OpenConnection(string str_connect_string)
	{
		if (string.IsNullOrEmpty(str_connect_string))
		{
			m_status.ErrorMessage = "No database connection string specified.";
			return null;
		}

		OleDbConnection conn;

		/// catch connection string syntax errors
		try
		{
			conn	= new OleDbConnection(str_connect_string);
		}
		catch (Exception ex)
		{
			m_status.ErrorMessage		= "Couldn't create database connection.";
			m_status.CurrentException	= ex;
			return null;
		}
		
		/// catch connection errors
		try
		{
			conn.Open();
		}
		catch (OleDbException olex)
		{
			m_status.ErrorMessage			= "Couldn't open database connection.";
			m_status.CurrentOleDbException = olex;
			return null;
		}
		catch (Exception ex)
		{
			m_status.CurrentException = ex;
			return null;
		}
		return conn;
	}


	public bool GetData(string str_connect_string)
	{
		string	str_cmd		= SqlCommand;

		if( string.IsNullOrEmpty( str_cmd))
		{
			m_status.ErrorMessage = "No data table or sql command specified.";
			return false;
		}

		if( UpdateSchema( str_connect_string) == false)
			return false;

		OleDbConnection		conn	= OpenConnection( str_connect_string);
		if( conn == null)
			return false;

		OleDbCommand		cmd		= new OleDbCommand( str_cmd, conn);
		OleDbDataReader		dr		= null;
		SilverDataRow		row;

		m_rows.Clear();

		try
		{
			dr = cmd.ExecuteReader(CommandBehavior.CloseConnection);
			while (dr.Read())
			{
				row		= new SilverDataRow(this);
				row.ReadDataReader( dr);
				m_rows.Add( row);
				if( m_rows.Count >= m_max_records )
					break;
			}
		}
		catch (OleDbException olex)
		{
			m_status.CurrentOleDbException = olex;
			return false;
		}
		catch (Exception ex)
		{
			m_status.CurrentException = ex;
			return false;
		}
		finally
		{
			if( dr != null)
				dr.Close();
			cmd.Dispose();
			conn.Dispose();
			OleDbConnection.ReleaseObjectPool();
		}

		return true;
	}


}
