﻿/// Copyright (c) Taoffi Nassar, 2009-2010. All Rights Reserved.
/// http://www.isosoft.org/taoffi
///
/// This code is provided as is for sample demonstration purpose
/// 
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Data;

/// <summary>
/// Summary description for SilverMetaTable
/// </summary>
[DataContract]
public class SilverMetaTable
{
	protected List<SilverMetaColumn>		m_columns	= new List<SilverMetaColumn>();

	public SilverMetaTable()
	{
	}


	[DataMember]
	public List<SilverMetaColumn> Columns
	{
		get
		{
			if( m_columns == null)
				m_columns	= new List<SilverMetaColumn>();

			return m_columns;
		}
		set
		{
			if (value == null)
				return;
			m_columns = value;
		}
	}

	public int Count
	{
		get
		{
			if( m_columns == null)
				return 0;
			return m_columns.Count;
		}
	}

	public SilverMetaColumn this[string column_name]
	{
		get
		{
			if( string.IsNullOrEmpty( column_name) || Count <= 0)
				return null;

			foreach (SilverMetaColumn col in m_columns)
			{
				if( string.Compare( col.Name, column_name, true) == 0)
					return col;
			}
			return null;
		}
	}

	public void Add(SilverMetaColumn meta_column)
	{
		if( meta_column == null || string.IsNullOrEmpty( meta_column.Name) || meta_column.DataType == null)
			return;

		/// does this column already exist?
		SilverMetaColumn	col	= this[meta_column.Name];

		if( col == null)
			Columns.Add(new SilverMetaColumn(meta_column));
		else
			col.CopyOther( meta_column);
	}


	public void Add(DataColumn data_column)
	{
		if( data_column == null || string.IsNullOrEmpty( data_column.ColumnName) || data_column.DataType == null)
			return;

		/// does this column already exist?
		SilverMetaColumn	col	= this[data_column.ColumnName];

		if( col == null)
			Columns.Add(new SilverMetaColumn(data_column));
		else
			col.ReadColumnInfo( data_column);
	}


	public bool ReadDbTableColumns(DataTable db_table)
	{
	    if( db_table == null || db_table.Columns == null)
	        return false;

	    Columns.Clear();

		foreach (DataColumn col in db_table.Columns)
		{
			Add( col);
		}

	    return true;
	}

}
