﻿/// Copyright (c) Taoffi Nassar, 2010. All Rights Reserved.
/// http://www.isosoft.org/taoffi
///
/// This code is provided as is for sample demonstration purpose
/// 
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

using SilverDbDataGrid.ServiceReference1;
using System.Windows.Data;


namespace SilverDbDataGrid
{
	public partial class MainPage : UserControl
	{

		protected SilverDataTable		m_table			= null;
		//protected SilverDatasetView		m_dataset_view	= new SilverDatasetView();


		public MainPage()
		{
			InitializeComponent();
		}

		private void button_get_data_Click(object sender, RoutedEventArgs e)
		{
			string		str_connet_string	= textbox_connect_string.Text,
						str_sql_command		= textbox_sql_command.Text,
						str_n_records		= textbox_n_records.Text;
			int			n_records			= 50;


			if (string.IsNullOrEmpty(str_connet_string) || string.IsNullOrEmpty(str_sql_command))
			{
				MessageBox.Show("Please provide the connetion string and the sql command.", "Error", MessageBoxButton.OK);
				return;
			}

			if (!string.IsNullOrEmpty(str_n_records))
			{
				int	tmp;
				if (int.TryParse(str_n_records, out tmp) == false)
				{
					MessageBox.Show("Please provide a numeric value for the number of records.", "Error", MessageBoxButton.OK);
					return;
				}

				n_records	= tmp;
			}

			DataServiceClient	cli	= service_helpers.DataServiceProxy(5);
			if (cli == null)
			{
				MessageBox.Show("Couldn't create the service proxy to communicate with the server.", "Service access", MessageBoxButton.OK);
				return;
			}
			Cursor	= Cursors.Wait;
			cli.GetDataCompleted += new EventHandler<GetDataCompletedEventArgs>(cli_GetDataCompleted);
			cli.GetDataAsync( str_connet_string, str_sql_command, n_records);
		}


		void cli_GetDataCompleted(object sender, GetDataCompletedEventArgs e)
		{
			Cursor = Cursors.Arrow;

			if (e.Cancelled)
			{
				MessageBox.Show("Request cancelled.", "DB access", MessageBoxButton.OK);
				return;
			}

			if ( e.Error != null)
			{
				MessageBox.Show("Service Access error: " + e.Error.Message, "DB access", MessageBoxButton.OK);
				return;
			}

			m_table	= (SilverDataTable) e.Result;

			if( m_table == null)
			{
				MessageBox.Show("Service returned null: " + e.Error.Message, "DB access", MessageBoxButton.OK);
				return;
			}

			if( ! string.IsNullOrEmpty( m_table.ErrorMessage))
			{
				MessageBox.Show(m_table.ErrorMessage, "DB access", MessageBoxButton.OK);
				return;
			}

			ConvertSilverData2DataGrid();
		}




		private void ConvertSilverData2DataGrid()
		{
			const string		row_binding_path	= "Cells",
								row_converter		= "row_converter";

			LayoutRoot.DataContext	= m_table;
			
			data_grid.Columns.Clear();

			if( m_table == null || m_table.MetaDataTable == null
					|| m_table.MetaDataTable.Columns == null
					|| m_table.MetaDataTable.Columns.Count <= 0)
				return;

			SilverMetaTable		meta_table	= m_table.MetaDataTable;
			int					col_index	= 0;

			//m_dataset_view.SourceCollection		= m_table.Rows;
			data_grid.ItemsSource				= m_table.Rows;	// m_dataset_view;

			foreach (SilverMetaColumn cell in meta_table.Columns)
			{
				DataGridBoundColumn		col		= GetDataGridColumn( cell);
				Binding					binding	= new Binding();

				col.Header					= cell.Caption;
				binding.Path				= new PropertyPath( row_binding_path);
				binding.Mode				= BindingMode.OneWay;
				binding.Converter			= (SilverRowConverter) this.Resources[row_converter];
				binding.ConverterParameter	= col_index;

				if( col is DataGridTextColumn == true)
					((DataGridTextColumn) col).Binding		= binding;
				if( col is DataGridCheckBoxColumn)
					((DataGridCheckBoxColumn)col).Binding	= binding;
				
				// preprae for column sorting on column header click
				col.CanUserSort		= true;
				col.SortMemberPath	= cell.Name;

				data_grid.Columns.Add(col);
				col_index++;
			}
		}


		private DataGridBoundColumn GetDataGridColumn(SilverMetaColumn meta_col)
		{
			DataGridTextColumn	text_col	= new DataGridTextColumn();
			
			text_col.MinWidth	= 50.0;

			if( meta_col == null)
				return text_col;

			switch ( meta_col.DataTypeName.ToLower())
			{
				case "bool":
				case "boolean":
					DataGridCheckBoxColumn col	= new DataGridCheckBoxColumn();
					return col;

				default:
					return text_col;
			}
		}

		private void UserControl_Loaded(object sender, RoutedEventArgs e)
		{
			textbox_connect_string.Focus();
		}


		private void data_grid_MouseWheel(object sender, MouseWheelEventArgs e)
		{
			/// If the mouse wheel delta is positive, move the box up.
			if (e.Delta > 0)
			{
				data_grid.ScrollDown();
			}

			/// If the mouse wheel delta is negative, move the box down.
			if (e.Delta < 0)
			{
				data_grid.ScrollUp();
			}
		}
	}
}
