﻿/// Copyright (c) Taoffi Nassar, 2010. All Rights Reserved.
/// http://www.isosoft.org/taoffi
///
/// This code is provided as is for sample demonstration purpose
/// 
using System;
using System.Windows.Browser;

using SilverDbDataGrid.ServiceReference1;

namespace SilverDbDataGrid
{
	public static class service_helpers
	{
		/// <summary>
		/// get config name (see ServiceReferences.ClientConfig)
		/// </summary>
		internal static string DataService_config_name
		{
			get { return "BasicHttpBinding_DataService"; }
		}

		/// <summary>
		/// get default data service url
		/// </summary>
		internal static string DataService_url
		{
			get
			{
				string service_url = BaseUrl;

				if (service_url.EndsWith("/") == false)
					service_url += "/";

				return service_url += "service/DataService.svc";
			}
		}


		public static DataServiceClient DataServiceProxy(int operation_timeout_minutes)
		{
			string	service_url		= DataService_url,
					config_name		= DataService_config_name;

			DataServiceClient cli = new DataServiceClient(config_name, service_url);

			cli.InnerChannel.OperationTimeout = TimeSpan.FromMinutes(operation_timeout_minutes);
			return cli;
		}

		/// <summary>
		/// get the current web site base url
		/// </summary>
		public static string BaseUrl
		{
			get
			{
				Uri				page_url		= HtmlPage.Document.DocumentUri;
				HtmlElement		ctrl			= HtmlPage.Document.GetElementById("base_url");
				string			host			= page_url.Host;
				string			str_http		= Url_server_base_address( page_url);

				if (ctrl != null)
				{
					string	str_val			= ctrl.GetAttribute("value");

					if (string.IsNullOrEmpty(str_val) == false)
					{
						if( str_val.EndsWith("/") == false)
							str_val += "/";

						return str_http + str_val;
					}
				}
				
				string	str_path		= page_url.AbsolutePath,
						str_app			= "",
						str_ret			= str_http;
				int		ndx				= str_path.IndexOf('/', 1);

				if ( ndx < 0)
					str_app = "/";
				else
					str_app = str_path.Substring(0, ndx);

				str_ret		= str_http + str_app;

				if (str_ret.EndsWith("/") != true)
					str_ret += "/";

				return str_ret;
			}
		}

		static string Url_server_base_address(Uri page_url)
		{
			if( page_url == null)
				return "";

			string	str_ret		= page_url.Scheme + "://" + page_url.Host;

			if (page_url.Port != 80)
				str_ret += ":" + page_url.Port.ToString();

			return str_ret;
		}

	}
}
